#coding=utf8

# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.

'''
CloudControl migrate command
'''

from cloudcontrol.cli.exception import *
from sjrpc.core.exceptions import *
from cloudcontrol.cli.printer import Printer, color
from cloudcontrol.cli.command import TqlCommand

class Command_migrate(TqlCommand):
    '''Migrate vm'''

    def __init__(self, cli, argv0):
        TqlCommand.__init__(self, cli, argv0)
        self.set_usage("%prog [options] <source tql> <dest tql>")
        self.remove_option("--direct")
        self.remove_option("--raw")
        self.remove_option("--print-tql")
        self.add_option("-l", "--list", action="store_true", dest="list", default=False,
                        help="List migration types and algo")
        self.add_option("-t", "--type", action="store", dest="type", default="",
                        help="Selection migration type")
        self.add_option("-a", "--algo", action="store", dest="algo", default="",
                        help="Select migration algorithm")

    def __call__(self, argv):
        # Parse argline
        self.parse_args(argv)
        # Retrieve election types
        self.etypes = self.get_electiontypes()
        # Check args and do listings
        if self.options.list:
            self.list()
            return
        elif self.options.type not in self.etypes:
            raise cmdBadArgument("No such type: %s"%self.options.type)
        elif self.options.algo not in self.etypes[self.options.type]:
            raise cmdBadArgument("No such algo: %s for type: %s"%(self.options.algo, self.options.type))
        elif len(self.args) != 2:
            raise cmdBadArgument()
        stql = self.args[0]
        dtql = self.args[1]
        # election(query_vm, query_dest, mtype='cold', algo='fair', **kwargs)
        scrutin = self.rpccall("election", stql, dtql,
                               mtype=self.options.type,
                               algo=self.options.algo,
                               _direct=True, _status=False)
        # check election result
        if len(scrutin) == 0:
            raise cmdError("No migration plan found")
        # print election
        for (i,o) in enumerate(scrutin):
            if self.options.index:
                self.printer.out("[%d] "%i, nl="")
            self.printer.out("%s%s %s-> %s%s%s (%s)"%(
                    self.tdc("id"),
                    o["sid"],
                    color["reset"],
                    self.tdc("id"),
                    o["did"],
                    color["reset"],
                    o["type"]))
        # ask confirmation
        if self.printer.ask("Do you confirm election? (Yes baby) ") != "Yes baby":
            raise cmdWarning("User resign")
        # run migration
        self.rpccall("migrate", scrutin, _direct=True)

    def remote_functions(self):
        return set(("migrate", "electiontypes", "election"))

    def get_electiontypes(self):
        '''Return a list of migration type'''
        try:
            return self.cli.rpc.call("electiontypes")
        except RpcError as e:
            raise cmdError(e)

    def list(self):
        '''Print a list of migration type'''
        for t in self.etypes.keys():
            self.printer.out("migrate -t %s -a %s"%(t, ",".join(self.etypes[t])))


class Command_migrate2(TqlCommand):
    '''New generation VM migration.'''

    def __init__(self, cli, argv0):
        TqlCommand.__init__(self, cli, argv0)
        self.set_usage("%prog [options] <source tql> <dest tql>")
        self.add_option("-l", "--live", action="store_true", default=False,
            help="Do a live migration")
        self.add_option("-b", "--batch", help="Name of this batch")
        self.add_option("-f", "--flag", action="append",
            help="Migration or allocation flags")

    def __call__(self, argv):
        self.parse_args(argv)
        if self.options.live:
            self.tql_filter += "&r=vm&status=running"
        else:
            self.tql_filter += "&r=vm&status=stopped"
        if len(self.args) != 2:
            raise cmdBadArgument()
        self.rpccall("migrate2", self.args[0], self.args[1],
            live=self.options.live, flags=self.options.flag)

    def remote_functions(self):
        return set(("migrate2",))
